import { MAX_TARGETS, TARGET_WIDTH } from './Constants';
import { Subject } from './Subject';
import { Gun } from './Gun';

import teddy from '../data/teddy.png';
import duck from '../data/duck.png';
import squirrel from '../data/squirrel.png';

class Target extends Subject {
  constructor(x, y, img) {
    super();
    this.x = x;
    this.y = y;
    this.img = img;
    this.visible = true;
  }

  draw() {
    if (this.visible && this.img) {
      image(this.img, this.x, this.y, TARGET_WIDTH, TARGET_WIDTH);
    }
  }

  getPoints() {
    return 0; // to be overridden
  }

  hit() {
    if (this.visible) {
      this.visible = false;
      this.notifySubscribers('target-hit', this.getPoints());
    }
  }

  contains(px, py) {
    return this.visible &&
      px >= this.x &&
      px <= this.x + TARGET_WIDTH &&
      py >= this.y &&
      py <= this.y + TARGET_WIDTH;
  }

  update(source, ...others) {
    if (source === 'gun') {
      const [x, y] = others;
      if (this.contains(x, y)) {
        this.hit();
      }
    }
  }
}

class TeddyTarget extends Target {
  constructor(x, y) {
    super(x, y, loadImage(teddy));
  }

  getPoints() {
    return 1;
  }
}

class DuckTarget extends Target {
  constructor(x, y) {
    super(x, y, loadImage(duck));
  }

  getPoints() {
    return 3;
  }
}

class SquirrelTarget extends Target {
  constructor(x, y) {
    super(x, y, loadImage(squirrel));
  }

  getPoints() {
    return 5;
  }
}

class TargetFactory {
  static instance;

  static getInstance() {
    if (!TargetFactory.instance) {
      TargetFactory.instance = new TargetFactory();
    }
    return TargetFactory.instance;
  }

  getTargetsByName(targetNames, targetWidth, y) {
    const targets = [];

    for (let i = 0; i < targetNames.length && i < MAX_TARGETS; i += 1) {
      const name = targetNames[i];
      const x = 100 + i * targetWidth;

      switch (name) {
        case 'teddy':
          targets.push(new TeddyTarget(x, y));
          break;
        case 'duck':
          targets.push(new DuckTarget(x, y));
          break;
        case 'squirrel':
          targets.push(new SquirrelTarget(x, y));
          break;
      }
    }

    return targets;
  }

  getRandomTargets(numTargets, targetWidth, y) {
    const names = ['teddy', 'duck', 'squirrel'];
    const targetNames = [];

    for (let i = 0; i < numTargets; i++) {
      targetNames.push(random(names));
    }

    return this.getTargetsByName(targetNames, targetWidth, y);
  }   // create random targets
}

export {
  Target, TeddyTarget,
  DuckTarget,
  SquirrelTarget, TargetFactory
};
